package com.caimei365.manager.FastDFS;

import com.caimei365.manager.utils.ImageUtils;
import com.github.tobato.fastdfs.domain.StorePath;
import com.github.tobato.fastdfs.proto.storage.DownloadByteArray;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URLEncoder;
import java.util.UUID;

@Slf4j
@Component
public class FastDFSClient {

    private static FastFileStorageClient storageClient;
    private static String config;
    private static String imageDomain;

    @Resource
    public void setStorageClient(FastFileStorageClient storageClient) {
        FastDFSClient.storageClient = storageClient;
    }

    @Value("${cm.config}")
    public void setConfig(String config) {
        FastDFSClient.config = config;
    }
    @Value("${caimei.imageDomain}")
    public void setImageDomain(String imageDomain) {
        FastDFSClient.imageDomain = imageDomain;
    }

    // 文件上传
    public static String uploadFile(String path) throws FileNotFoundException {
        File file = new File(path);
        InputStream input = new FileInputStream(file);
        long size = FileUtils.sizeOf(file);
        String name = file.getName();
        String fileName = name.substring(name.lastIndexOf("/") + 1);
        StorePath storePath = storageClient.uploadFile(input, size, FilenameUtils.getExtension(fileName), null);
        return storePath.getFullPath();
    }

    // 文件下载
    public static boolean downloadFile(String path, String downloadFilePath) throws IOException {
        File file = new File(downloadFilePath);
        FileOutputStream outputStream = null;
        // fastdfs 文件读取
        String filepath = path.substring(path.lastIndexOf("group1/") + 7);
        DownloadByteArray callback = new DownloadByteArray();
        byte[] content = storageClient.downloadFile("group1", filepath, callback);
        // 数据写入指定文件夹中
        outputStream = new FileOutputStream(file);
        outputStream.write(content);
        return true;
    }

    /**
     * 保存文件到FastDFS
     */
    public static String saveFile(ByteArrayInputStream inputStream) throws IOException {
        String randomStr = UUID.randomUUID().toString();
        String name = ".jpeg";
        // 图片暂存本地服务器路径
        String filePath = "/mnt/newdatadrive/data/runtime/jar-instance/manager-api/tempImage/";
        if ("dev".equals(config)) {
            filePath = "D:\\";
        }
        filePath += randomStr + name;
        FileOutputStream outputStream = new FileOutputStream(filePath);
        int i = 0;
        byte[] buffer = new byte[200];
        while ((i = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, i);
        }
        outputStream.flush();
        outputStream.close();
        inputStream.close();
        // 临时图片
        File tempFile = new File(filePath);
        log.info("【图片上传】>>>>>>>>>>>>>>>>图片临时路径:" + filePath);
        String fileUrl = uploadFile(filePath);
        // 删除临时图片
        boolean delete = tempFile.delete();
        log.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片:" + delete);
        return imageDomain + "/" +fileUrl;
    }

    /**
     * 保存文件到FastDFS
     */
    public static String saveImage(BufferedImage image) {
        try {
            return saveFile(new ByteArrayInputStream(ImageUtils.bufImg2Bytes(image)));
        } catch (IOException e) {
            log.error("【图片上传异常】>>>>>>>>>>>>>>>>:" + e);
            return null;
        }
    }

    public static void download(HttpServletRequest request, HttpServletResponse response, String filePath, String fileName) throws IOException {
            File file = new File(filePath);
            //  文件存在才下载
            if (file.exists()) {
            OutputStream out = null;
            FileInputStream in = null;
            try {
                // 1.读取要下载的内容
                in = new FileInputStream(file);
                // 2. 告诉浏览器下载的方式以及一些设置
                // 解决文件名乱码问题，获取浏览器类型，转换对应文件名编码格式，IE要求文件名必须是utf-8, firefo要求是iso-8859-1编码
                String agent = request.getHeader("user-agent");
                if (agent.contains("FireFox")) {
                    fileName = new String(fileName.getBytes("UTF-8"), "iso-8859-1");
                } else {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                }
                // 设置下载文件的mineType，告诉浏览器下载文件类型
                String mineType = request.getServletContext().getMimeType(fileName);
                response.setContentType(mineType);
                // 设置一个响应头，无论是否被浏览器解析，都下载
                response.setHeader("Content-disposition", "attachment; filename=" + fileName);
                // 将要下载的文件内容通过输出流写到浏览器
                out = response.getOutputStream();
                int len = 0;
                byte[] buffer = new byte[1024];
                while ((len = in.read(buffer)) > 0) {
                    out.write(buffer, 0, len);
                }
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
                file.delete();
            }
        }
    }



}
