package com.caimei365.manager.controller.caimei;

import cn.hutool.core.date.DateTime;
import com.caimei365.manager.config.utils.DateUtil;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.KeyWord;
import com.caimei365.manager.entity.caimei.productArchive.CmProductArchive;
import com.caimei365.manager.service.caimei.HomeService;
import com.github.pagehelper.PageHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 后台首页
 *
 * @author : Charles
 * @date : 2022/3/23
 */
@Validated
@RestController
@RequestMapping("/home")
public class HomeApi {
    @Resource
    private HomeService homeService;

    /**
     * 仪表盘数据
     */
    @GetMapping("/dashboard")
    public ResponseJson<Map<String, Object>> getDashboardData() {
        return homeService.getDashboardData();
    }

    /**
     * 机构统计
     */
    @GetMapping("/getClubCount")
    public ResponseJson<Map<String, Object>> getClubCount(Date startCreateTime, Date endCreateTime) {
        if (null != endCreateTime) {
            endCreateTime = DateUtil.getEndTime(endCreateTime);
        }
        return homeService.getClubCount(startCreateTime, endCreateTime);
    }

    /**
     * 机构趋势
     */
    @GetMapping("/getClubTrend")
    public ResponseJson<Map<String, Object>> getClubTrend(Date startCreateTime, Date endCreateTime) {
        if (null != endCreateTime) {
            endCreateTime = DateUtil.getEndTime(endCreateTime);
        }
        return homeService.getClubTrend(startCreateTime, endCreateTime);
    }

    /**
     * 访问统计
     */
    @GetMapping("/getAccessRecordCount")
    public ResponseJson<Map<String, Object>> getAccessRecordCount(Date startCreateTime, Date endCreateTime) {
        if (null != endCreateTime) {
            endCreateTime = DateUtil.getEndTime(endCreateTime);
        }
        return homeService.getAccessRecordCount(startCreateTime, endCreateTime);
    }

    /**
     * 关键词下拉选择项
     */
    @GetMapping("/findKeyWordList")
    public ResponseJson findKeyWordList(String keyword,
                                        @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                        @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        return ResponseJson.success(new PaginationVo(homeService.findKeyWordList(keyword)));
    }

    /**
     * 已上线商品列表
     */
    @GetMapping("/findProductList")
    public ResponseJson findProductList(String shopName,
                                        String productName,
                                        Integer productId,
                                        @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                        @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmProductArchive> list = homeService.findProductList(new CmProductArchive()
                .setProductId(productId)
                .setProductName(productName)
                .setShopName(shopName)
        );
        return ResponseJson.success(new PaginationVo(list));
    }

}
