package com.caimei365.manager.controller.caimei.providers;

import com.caimei365.manager.FastDFS.FastDFSClient;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.CmRelatedImage;
import com.caimei365.manager.entity.caimei.providers.CmProviders;
import com.caimei365.manager.entity.caimei.providers.CmProvidersContract;
import com.caimei365.manager.service.caimei.providers.CmProvidersContractService;
import com.caimei365.manager.service.caimei.providers.CmProvidersService;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 功能描述:服务商入驻Controller
 *
 * @auther: Kaick
 * @date: 2023/12/13 15:30
 */
@Validated
@RestController
@RequestMapping("/providers")
public class ProvidersApi {
    @Resource
    private CmProvidersService cmProvidersService;
    @Resource
    private CmProvidersContractService cmProvidersContractService;

    @GetMapping("/list")
    public ResponseJson providersList(String name,
                                      Integer serviceLevel,
                                      Integer serviceCategory,
                                      String qualificationId,
                                      Date contractStartDate,
                                      Date contractEndDate,
                                      @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                      @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmProviders> cmProvidersList = cmProvidersService.getCmProvidersList(new CmProviders()
                .setName(name)
                .setServiceLevel(serviceLevel)
                .setServiceCategory(serviceCategory)
                .setQualificationId(qualificationId)
                .setContractStartDate(contractStartDate)
                .setContractEndDate(contractEndDate)
        );
        return ResponseJson.success(new PaginationVo(cmProvidersList));
    }

    @GetMapping("/contract/list")
    public ResponseJson providersContractList(String contractId,
                                              @NotNull(message = "服务商ID不能为空") Integer providersId,
                                              Date contractStartDate,
                                              Date contractEndDate,
                                              @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                              @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmProvidersContract> cmProvidersContractList = cmProvidersContractService.getCmProvidersContractList(new CmProvidersContract()
                .setContractId(contractId)
                .setProvidersId(providersId)
                .setContractStartDate(contractStartDate)
                .setContractEndDate(contractEndDate)
        );
        return ResponseJson.success(new PaginationVo(cmProvidersContractList));
    }

    @GetMapping("/from")
    public ResponseJson providersFrom(String providersId) {
        if (StringUtils.isBlank(providersId)
        ) {
            return ResponseJson.error("参数异常！");
        }
        CmProviders providers = cmProvidersService.getCmProvidersById(providersId);
        return ResponseJson.success(providers);
    }

    @GetMapping("/contract/from")
    public ResponseJson providersContractFrom(String providersContractId) {
        if (StringUtils.isBlank(providersContractId)
        ) {
            return ResponseJson.error("参数异常！");
        }
        CmProvidersContract providersContract = cmProvidersContractService.getCmProvidersContractById(providersContractId);
        return ResponseJson.success(providersContract);
    }

    @PostMapping("/add")
    public ResponseJson providersAdd(@RequestBody CmProviders cmProviders
    ) {
        if (null == cmProviders.getName() ||
                null == cmProviders.getLinkMan() ||
                null == cmProviders.getMobile() ||
                null == cmProviders.getServiceLevel() ||
                null == cmProviders.getServiceCategory() ||
                null == cmProviders.getAddress()||
                null == cmProviders.getTextRelateds()||
                null == cmProviders.getInfoRelateds()
        ) {
            return ResponseJson.error("参数异常！");
        }
        cmProvidersService.addCmProviders(cmProviders);
        return ResponseJson.success();
    }

    @PostMapping("/updateStatus")
    public ResponseJson providersUpdateStatus(String userIds, Integer status) {
        if (null == userIds) {
            return ResponseJson.error("ID不能为空！");
        }
        for (String userId : userIds.split(",")) {
            cmProvidersService.updateCmProviders(new CmProviders()
                    .setUserId(Integer.valueOf(userId))
                    .setStatus(status == 90 ? 90 : 91)
            );
        }
        return ResponseJson.success();
    }

    @PostMapping("/contract/add")
    public ResponseJson providersContractAdd(@RequestBody CmProvidersContract cmProvidersContract) {
        if (null == cmProvidersContract.getProvidersId() ||
                null == cmProvidersContract.getContractId() ||
                null == cmProvidersContract.getContractStartDate() ||
                null == cmProvidersContract.getContractEndDate() ||
                null == cmProvidersContract.getContractRelateds()
        ) {
            return ResponseJson.error("参数异常！");
        }
        cmProvidersContractService.addCmProvidersContract(cmProvidersContract);
        return ResponseJson.success();
    }

    @GetMapping("/download/image")
    public void downloadImage(String ids, Integer type, HttpServletRequest request, HttpServletResponse response) throws Exception {
        if(StringUtils.isBlank(ids)||null==type){
            return;
        }
        cmProvidersService.downloadImage(ids, type, request, response);
    }
}
