package com.caimei365.manager.controller.caimei.user;

import com.caimei.utils.StringUtil;
import com.caimei.utils.StringUtils;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.CmShop;
import com.caimei365.manager.entity.caimei.CmShopLabel;
import com.caimei365.manager.entity.caimei.CmShopRelevance;
import com.caimei365.manager.service.caimei.user.CmShopLabelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/3/15
 */
@RestController
@RequestMapping("/user/shopLabel")
public class CmShopLabelApi {

    @Autowired private CmShopLabelService shopLabelService;

    /**
     * 关联标签供应商列表
     * @param cmShopRelevance 搜索条件
     * @param pageNum 页码
     * @param pageSize 页面数据数
     * @return 返回
     */
    @GetMapping("/shopRelevanceList")
    public ResponseJson<PaginationVo<CmShopRelevance>> shopRelevanceList(CmShopRelevance cmShopRelevance,
                                                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                         @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {
        return shopLabelService.pageList(cmShopRelevance,pageNum,pageSize);
    }

    /**
     * 供应商列表
     * @param shopId 供应商id
     * @param name 供应商名称
     * @param pageNum 页码
     * @param pageSize 页面数据数
     * @return 返回
     */
    @GetMapping("/shopList")
    public ResponseJson<PaginationVo<CmShop>> shopList(String shopId, String name,
                                                       @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                       @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {
        return shopLabelService.shopList(shopId,name,pageNum,pageSize);
    }

    /**
     * 添加供应商
     * @param shopId 供应商id
     * @return 返回
     */
    @GetMapping("/insertShopRelevance")
    public ResponseJson insertShopRelevance(Integer shopId) {
        if (shopId == null) {
            return ResponseJson.error("供应商Id不能为空",null);
        }
        return shopLabelService.insertShopRelevance(shopId);
    }

    /**
     * 删除关联标签的供应商
     * @param relevanceId 供应商id
     * @return 返回
     */
    @GetMapping("/delShopById")
    public ResponseJson delShopRelevanceById(Integer relevanceId) {
        if (relevanceId == null) {
            return ResponseJson.error("供应商Id不能为空",null);
        }
        return shopLabelService.delShopRelevanceById(relevanceId);
    }

    /**
     * 供应商关联标签列表
     * @param relevanceId 供应商id
     * @param keyword 标签名
     * @param pageNum 页码
     * @param pageSize 页面数据数
     * @return 返回
     */
    @GetMapping("/shopLabelList")
    public  ResponseJson<PaginationVo<CmShopLabel>> shopLabelList (Integer relevanceId, String keyword,
                                                                   @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                   @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {
        if (relevanceId == null) {
            ResponseJson.error("供应商Id不能为空",null);
        }
        return shopLabelService.shopLabelList(relevanceId,keyword,pageNum,pageSize);
    }

    /**
     * 标签列表
     * @param shopId 供应商Id
     * @param keyword 标签名
     * @param pageNum 页码
     * @param pageSize 页面数据数
     * @return 返回
     */
    @GetMapping("/labelList")
    public ResponseJson<PaginationVo<CmShopLabel>> labelList(String shopId, String keyword,
                                                             @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                             @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {

        return shopLabelService.labelList(shopId, keyword, pageNum, pageSize);
    }

    /**
     * 关联标签
     * @param relevanceId 供应商id
     * @param keywordIds 标签id或id集合
     * @return 返回
     */
    @GetMapping("/insertLabel")
    public ResponseJson insertLabel(Integer relevanceId, String keywordIds) {
        if (relevanceId == null) {
            return  ResponseJson.error("关联供应商Id不能为空", null);
        }
        if (!StringUtils.isNotBlank(keywordIds)) {
            return ResponseJson.error("标签Id不能为空",null);
        }
        return shopLabelService.insertLabel(relevanceId, keywordIds);
    }

    /**
     * 添加标签到标签库
     * @param keyword 标签
     * @return 返回
     */
    @GetMapping("/insertSearch")
    public ResponseJson insertSearch(String keyword) {
        if (!StringUtils.isNotBlank(keyword)) {
            return  ResponseJson.error("标签不能为空", null);
        }
        return shopLabelService.insertSearch(keyword);
    }

    /**
     * 删除供应商关联标签
     * @param labelIds id或id集合
     * @return 返回
     */
    @GetMapping("/delShopLabelList")
    public ResponseJson upShopLabelList(String labelIds) {

        if (!StringUtils.isNoneBlank(labelIds)) {
            return ResponseJson.error("Id不能为空",null);
        }
        return shopLabelService.upShopLabelList(labelIds);
    }
}
