package com.caimei365.manager.dao;

import java.util.List;

import com.caimei365.manager.entity.caimei.CmRelatedImage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * 相关图片Mapper接口
 *
 * @author Kaick
 * @date 2023-12-12
 */
@Mapper
public interface CmRelatedImageMapper
{
    /**
     * 通过对象查询相关图片列表
     *
     * @param cmRelatedImage 相关图片
     * @return 相关图片集合
     */
    List<CmRelatedImage> getRelatedImageList(CmRelatedImage cmRelatedImage);



    /**
     * 通过Id查询相关图片对象
     *
     * @param id 相关图片主键
     * @return 相关图片
     */
    CmRelatedImage getCmRelatedImageById(String id);

    /**
     * 通过对象查询相关图片对象
     *
     * @param cmRelatedImage 相关图片
     * @return 相关图片
     */
    CmRelatedImage getByCmRelatedImage(CmRelatedImage cmRelatedImage);

    /**
     * 通过对象查询相关图片Id
     *
     * @param cmRelatedImage 相关图片
     * @return String
     */
    String getById(CmRelatedImage cmRelatedImage);


    /**
     * 通过对象查询相关图片记录总数
     *
     * @param cmRelatedImage 相关图片
     * @return 相关图片Integer
     */
    int getCount(CmRelatedImage cmRelatedImage);

    /**
     * 新增相关图片
     *
     * @param cmRelatedImage 相关图片
     * @return 结果
     */
    int addCmRelatedImage(CmRelatedImage cmRelatedImage);

    /**
     * 修改相关图片
     *
     * @param cmRelatedImage 相关图片
     * @return 结果
     */
    int updateCmRelatedImage(CmRelatedImage cmRelatedImage);

    /**
     * 删除相关图片
     *
     * @param id 相关图片主键
     * @return 结果
     */
    int delCmRelatedImageById(String id);

    /**
     * 批量删除相关图片
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int delCmRelatedImage(CmRelatedImage cmRelatedImage);

}
