package com.caimei365.manager.service.caimei.impl;

import cn.hutool.core.lang.Assert;
import com.caimei.utils.StringUtils;
import com.caimei365.manager.dao.KeyWordDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.CmKeywordInfo;
import com.caimei365.manager.entity.caimei.CmKeywordSubtag;
import com.caimei365.manager.entity.caimei.CmPriorKeyword;
import com.caimei365.manager.entity.caimei.KeyWord;
import com.caimei365.manager.entity.caimei.cmUser.ServiceProviderModel;
import com.caimei365.manager.entity.caimei.enums.KeyWordEnum;
import com.caimei365.manager.service.caimei.KeyWordService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;


import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zzj
 */
@Slf4j
@Service
public class KeyWordServiceImpl implements KeyWordService {


    @Resource
    private KeyWordDao keyWordDao;

    @Override
    public ResponseJson<PaginationVo<KeyWord>> getKeyWordList(String keyword, Integer searchTimeCode, Integer fromSearch, String beginTime, String endTime, Integer labelStatus, int pageNum, int pageSize) {
        if (null != searchTimeCode && searchTimeCode > 0 && StringUtils.isEmpty(beginTime) && StringUtils.isEmpty(endTime)) {
            Map<String, String> timeVal = KeyWordEnum.getTimeVal(searchTimeCode);
            beginTime = timeVal.get("beginTime");
            endTime = timeVal.get("endTime");
        }
        if (!StringUtils.isEmpty(beginTime)) {
            beginTime += " 00:00:00";
        }
        if (!StringUtils.isEmpty(endTime)) {
            endTime += " 23:59:59";
        }
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> keyWords = keyWordDao.findList(keyword, fromSearch, beginTime, endTime, labelStatus);
        PaginationVo<KeyWord> pageData = new PaginationVo<>(keyWords);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson findLinkageStatus(String id, Integer linkageStatus) {
        String linkageText = "";
        if(linkageStatus==1){
            Map<String, String> linkageMap = keyWordDao.findLinkageStatus(id);
            if (null == linkageMap || null == linkageMap.get("product")) {
                linkageText += ",商品";
            }
            if (null == linkageMap || null == linkageMap.get("cm_product_archive")) {
                linkageText += ",美业资料";
            }
            if (null == linkageMap || null == linkageMap.get("info")) {
                linkageText += ",信息平台";
            }
            if (null == linkageMap || null == linkageMap.get("cm_baike_product")) {
                linkageText += ",采美百科";
            }
        }
        if(linkageStatus==1){
            if ("".equals(linkageText)) {
                keyWordDao.updateCmUserSearchFrequency(new KeyWord()
                        .setId(Integer.valueOf(id))
                        .setLinkageStatus(linkageStatus));
                linkageText = "关联成功";
            } else {
                linkageText = "该标签需关联上" + linkageText.substring(1) + "后才可开启";
            }
        }else {
            keyWordDao.updateCmUserSearchFrequency(new KeyWord()
                    .setId(Integer.valueOf(id))
                    .setLinkageStatus(linkageStatus));
            linkageText = "取消关联成功";
        }
        return ResponseJson.success(linkageText, linkageStatus);

    }


    @Override
    public ResponseJson joinLabel(String id) {
        Assert.notBlank(id);
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                keyWordDao.updateLabelStatus(s);
            }
        } else {
            keyWordDao.updateLabelStatus(id);
        }
        return ResponseJson.success();
    }

    @Override
    public ResponseJson<PaginationVo<KeyWord>> getRecommendList(String keyword, String beginTime, String endTime, Integer pageNum, Integer pageSize) {
        if (!StringUtils.isEmpty(beginTime)) {
            beginTime += " 00:00:00";
        }
        if (!StringUtils.isEmpty(endTime)) {
            endTime += " 23:59:59";
        }
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> keyWords = keyWordDao.findRecommendList(keyword, beginTime, endTime);
        PaginationVo<KeyWord> pageData = new PaginationVo<>(keyWords);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson ignoreLabel(String id) {
        Assert.notBlank(id);
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                keyWordDao.updateRecommendFlag(s);
            }
        } else {
            keyWordDao.updateRecommendFlag(id);
        }
        return ResponseJson.success();
    }

    @Override
    public ResponseJson insertLabel(KeyWord keyWord) {
        if (keyWord.getId() == null) {
            Integer keywordExist = keyWordDao.findKeywordExist(keyWord.getKeyword());
            if (null == keywordExist) {
                keyWordDao.insertLabel(keyWord.getKeyword());
                return ResponseJson.success();
            } else {
                return ResponseJson.error(-1, "关键词已存在", null);
            }
        } else {
            keyWordDao.updateCmUserSearchFrequency(keyWord);
            return ResponseJson.error(-1, "关键词已存在", null);
        }
    }

    @Override
    public ResponseJson deleteLabel(String id) {
        Assert.notBlank(id);
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                keyWordDao.updateDelFlag(s);
                keyWordDao.delPriorKeywordById(Integer.parseInt(s));
            }
        } else {
            keyWordDao.updateDelFlag(id);
        }
        return ResponseJson.success();
    }

    /**
     * 协销列表
     *
     * @return
     */
    @Override
    public ResponseJson<List<ServiceProviderModel>> getServiceList() {
        return ResponseJson.success(keyWordDao.getServiceList());
    }

    /**
     * 动态/静态标签列表
     * @param dynamicStatus
     * @param parentLabel
     * @param keyword
     * @param fromSearch
     * @param beginTime
     * @param endTime
     * @param serviceProviderId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmKeywordInfo>> getStateKeyWordList(Integer dynamicStatus, Integer parentLabel, String keyword, Integer fromSearch, String beginTime, String endTime, Integer serviceProviderId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmKeywordInfo> keyWords = keyWordDao.getStateKeyWordList(dynamicStatus, parentLabel, keyword, fromSearch, beginTime, endTime, serviceProviderId);
        PaginationVo<CmKeywordInfo> pageData = new PaginationVo<>(keyWords);
        return ResponseJson.success(pageData);
    }

    /**
     * 动态标签编辑回显
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson<CmKeywordInfo> editKeyword(Integer id) {
        // 标签数据
        CmKeywordInfo cmKeywordInfo = keyWordDao.getkeywordById(id);
        if (null != cmKeywordInfo) {
            // 前端要求默认 【】
            cmKeywordInfo.setSubtag(new ArrayList<>());
            // 若为父标签，获取子标签返回
            if (1 == cmKeywordInfo.getParentLabel()) {
                List<CmKeywordSubtag> cmKeywordSubtags = keyWordDao.getkeywordSubtag(id);
                cmKeywordInfo.setSubtag(null == cmKeywordSubtags ? new ArrayList<>() : cmKeywordSubtags);
            }
        } else {
            return ResponseJson.error(-1, "标签数据异常", null);
        }
        return ResponseJson.success(cmKeywordInfo);
    }

    /**
     * 选择子标签
     *
     * @param keyword
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmKeywordInfo>> getSubtagKeyword(String keyword, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmKeywordInfo> keywordList = keyWordDao.getKeyword(keyword);
        PaginationVo<CmKeywordInfo> page = new PaginationVo<>(keywordList);
        return ResponseJson.success(page);
    }

    /**
     * 保存 动态/静态标签
     *
     * @param cmKeywordInfo
     * @return
     */
    @Override
    public ResponseJson insertKeywordInfo(CmKeywordInfo cmKeywordInfo) {
        List<Integer> ids = new ArrayList<>();
        if (null == cmKeywordInfo.getId()) {
            // 添加
            Integer key = keyWordDao.getKeywordByName(cmKeywordInfo.getKeyword());
            // 添加标签
            if (null == key) {
                keyWordDao.insertKeywordInfo(cmKeywordInfo);
            } else {
                return ResponseJson.error(-1, "标签已存在", null);
            }
        } else {
            // 修改
            keyWordDao.updateKeyword(cmKeywordInfo);
        }
        // 有子标签时将子标签与父标签关联
        if (null != cmKeywordInfo.getSubtag() && cmKeywordInfo.getSubtag().size() > 0) {
            for (CmKeywordSubtag subtag : cmKeywordInfo.getSubtag()) {
                if (null == subtag.getId()) {
                    return ResponseJson.error(-1, "子标签id不能为空", null);
                }
                subtag.setKeywordId(cmKeywordInfo.getId());
                subtag.setSubtagId(subtag.getSubtagId());
                keyWordDao.insertKeywordSubtag(subtag);
                ids.add(subtag.getId());
            }
            log.info("ids==="+ids);
            if (null != ids && ids.size() >0) {
                // 删除与父标签关联
                keyWordDao.updateKeywordSubtag(cmKeywordInfo.getId(), ids);
            }
        }
        return ResponseJson.success();
    }

    /**
     * 删除动态标签
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson deleteTrendsLabel(String id) {
        if (id.contains(",")) {
            String[] split = id.split(",");
            for (String s : split) {
                // 删除动态/静态标签
                keyWordDao.updateKeywordInfo(s);
                // 删除有限展示标签
                keyWordDao.delPriorKeywordById(Integer.parseInt(s));
                // 删除子标签
                keyWordDao.updateKeywordSubtag(Integer.parseInt(s), new ArrayList<>());
            }
        } else {
            // 删除动态/静态标签
            keyWordDao.updateKeywordInfo(id);
            // 删除有限展示标签
            keyWordDao.delPriorKeywordById(Integer.parseInt(id));
            // 删除子标签
            keyWordDao.updateKeywordSubtag(Integer.parseInt(id), new ArrayList<>());
        }
        return ResponseJson.success();
    }

    /**
     * 优先展示标签列表
     *
     * @param keyword
     * @param fromSearch
     * @param dynamicStatus
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmPriorKeyword>> getPriorKeywordList(String keyword, Integer fromSearch, Integer dynamicStatus, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmPriorKeyword> priorKeywordList = keyWordDao.getPriorKeywordList(keyword, fromSearch, dynamicStatus);

        PaginationVo<CmPriorKeyword> page = new PaginationVo<>(priorKeywordList);
        return ResponseJson.success(page);
    }

    /**
     * 选择标签
     *
     * @param keyword
     * @param fromSearch
     * @param dynamicStatus 0 为动态标签 1 静态标签
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<KeyWord>> getPickKeyword(String keyword, Integer fromSearch, Integer dynamicStatus, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<KeyWord> priorKeywordList = keyWordDao.getPickKeyword(keyword, fromSearch, dynamicStatus);
        List<CmPriorKeyword> keywordList = keyWordDao.getPriorKeywordList("",null,null);
        List<Integer> searchIds = new ArrayList<>();
        if (null != keywordList && keywordList.size()>0) {
            searchIds = keywordList.stream().map(CmPriorKeyword::getSearchId).collect(Collectors.toList());
        }
        for (KeyWord key : priorKeywordList) {
            if (searchIds.contains(key.getId())) {
                key.setFlag(false);
            } else {
                key.setFlag(true);
            }
        }
        PaginationVo<KeyWord> page = new PaginationVo<>(priorKeywordList);
        return ResponseJson.success(page);
    }

    /**
     * 保存优先展示标签
     *
     * @param searchIds
     * @return
     */
    @Override
    public ResponseJson savePriorKeyword(String searchIds) {
        List<Integer> searchIdList = new ArrayList<>();
        if (searchIds.contains(",")) {
            String[] split = searchIds.split(",");
            for (String s : split) {
                searchIdList.add(Integer.parseInt(s));
            }
        } else {
            searchIdList.add(Integer.parseInt(searchIds));
        }
        Integer count = keyWordDao.getPriorCount();
        if (count > 50) {
            return ResponseJson.error(-1, "只支持展示50个标签，请删减再添加。", null);
        }
        if ((count + searchIdList.size()) > 50) {
            int num = (count + searchIdList.size()) - 50;
            return ResponseJson.error(-1, "只支持展示50个标签，已超出"+ num +"(个)，请删减再添加。", null);
        }
        // 保存优先展示标签
        for (Integer id : searchIdList) {
            Integer prior = keyWordDao.getPrior(id);
            if (null == prior) {
                keyWordDao.insertPriorKeyword(id);
            } else {
                return ResponseJson.error(-1, "关键词已存在", null);
            }
        }
        return ResponseJson.success();
    }

    /**
     * 删除优先展示标签
     *
     * @param ids
     * @return
     */
    @Override
    public ResponseJson delPriorKeyword(String ids) {
        if (ids.contains(",")) {
            String[] split = ids.split(",");
            for (String id : split) {
                keyWordDao.delPriorKeyword(Integer.parseInt(id));
            }
        } else {
            keyWordDao.delPriorKeyword(Integer.parseInt(ids));
        }
        return ResponseJson.success();
    }
}
