package com.caimei365.manager.service.caimei.productArchive.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.caimei365.manager.dao.productArchive.CmProductArchiveContentMapper;
import com.caimei365.manager.dao.productArchive.CmProductArchiveFileMapper;
import com.caimei365.manager.entity.caimei.productArchive.CmProductArchiveContent;
import com.caimei365.manager.entity.caimei.productArchive.CmProductArchiveFile;
import com.caimei365.manager.service.caimei.impl.AsyncService;
import com.caimei365.manager.service.caimei.productArchive.CmProductArchiveContentService;
import com.caimei365.manager.utils.WaterMarkUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;


import javax.annotation.Resource;

/**
 * 商品资料内容Service业务层处理
 *
 * @author Kaick
 * @date 2023-12-22
 */
@Service
public class CmProductArchiveContentServiceImpl implements CmProductArchiveContentService {
    @Resource
    private CmProductArchiveContentMapper cmProductArchiveContentMapper;
    @Resource
    private CmProductArchiveFileMapper cmProductArchiveFileMapper;
    @Resource
    private AsyncService asyncService;
    /**
     * 通过对象查询商品资料内容列表
     *
     * @param cmProductArchiveContent 商品资料内容
     * @return 商品资料内容
     */
    @Override
    public List<CmProductArchiveContent> getCmProductArchiveContentList(CmProductArchiveContent cmProductArchiveContent) {
        return cmProductArchiveContentMapper.getCmProductArchiveContentList(cmProductArchiveContent);
    }

    /**
     * 通过对象查询商品资料内容
     *
     * @param cmProductArchiveContent 商品资料内容
     * @return 商品资料内容
     */
    @Override
    public CmProductArchiveContent getCmProductArchiveContent(CmProductArchiveContent cmProductArchiveContent) {
        CmProductArchiveContent byCmProductArchiveContent = cmProductArchiveContentMapper.getByCmProductArchiveContent(cmProductArchiveContent);
        if (byCmProductArchiveContent != null) {
            byCmProductArchiveContent.setArchiveFiles(cmProductArchiveFileMapper.getCmProductArchiveFileList(new CmProductArchiveFile().setArchiveContentId(Integer.valueOf(byCmProductArchiveContent.getId()))));
        }
        return byCmProductArchiveContent;
    }


    /**
     * 新增商品资料内容
     *
     * @param cmProductArchiveContent 商品资料内容
     * @return 结果
     */
    @Override
    public int addCmProductArchiveContent(CmProductArchiveContent cmProductArchiveContent) {
        if (StringUtils.isBlank(cmProductArchiveContent.getId())) {
            cmProductArchiveContent.setAddTime(new Date());
            cmProductArchiveContentMapper.addCmProductArchiveContent(cmProductArchiveContent);
        } else {
            cmProductArchiveContentMapper.updateCmProductArchiveContent(cmProductArchiveContent);
            cmProductArchiveFileMapper.delCmProductArchiveFile(new CmProductArchiveFile().setArchiveContentId(Integer.valueOf(cmProductArchiveContent.getId())));
        }
        if (StringUtils.equals("5", cmProductArchiveContent.getType()) ||
                StringUtils.equals("6", cmProductArchiveContent.getType())) {
            return 1;
        }
        for (CmProductArchiveFile archiveFile : cmProductArchiveContent.getArchiveFiles()) {
            archiveFile.setArchiveContentId(Integer.valueOf(cmProductArchiveContent.getId()));
            archiveFile.setUploadTime(new Date());
            cmProductArchiveFileMapper.addCmProductArchiveFile(archiveFile);
            asyncService.setWaterOss(archiveFile);
        }
        return 1;
    }

    public int delCmProductArchiveContent(CmProductArchiveContent cmProductArchiveContent) {
        return cmProductArchiveContentMapper.delCmProductArchiveContent(cmProductArchiveContent);
    }


}



