package com.caimei365.manager.service.caimei.user.impl;

import com.caimei365.manager.dao.user.UserMapper;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.CmBehaviorRecord;
import com.caimei365.manager.entity.caimei.CmShopOrder;
import com.caimei365.manager.entity.caimei.cmUser.*;
import com.caimei365.manager.service.caimei.user.CmUserService;
import com.github.pagehelper.PageHelper;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class CmUserServiceImpl implements CmUserService {

    @Resource
    private UserMapper userMapper;

    @Override
    public ResponseJson<PaginationVo<CmClub>> getClubList(CmClub club, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmClub> clubs = userMapper.findClubList(club);
        PaginationVo<CmClub> pageList = new PaginationVo<>(clubs);
        return ResponseJson.success(pageList);
    }

    @Override
    public ResponseJson<List<CmClubLabel>> getClubLabels(Integer dynamicStatus) {
        List<CmClubLabel> labels = userMapper.findLabels(dynamicStatus);
        return ResponseJson.success(labels);
    }

    @Override
    public ResponseJson<CmPortrait> getClubPortrait(Integer clubId) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        CmPortrait clubStatistics = userMapper.selClubData(clubId);
        if (clubStatistics != null) {
            String type = "";
            if (StringUtils.isNotBlank(clubStatistics.getFirstClubType())) {
                type += clubStatistics.getFirstClubType();
            }
            if (StringUtils.isNotBlank(clubStatistics.getClassify())) {
                type += "-" + clubStatistics.getClassify();
            }
            clubStatistics.setClubType(type);
            setClubNum(clubStatistics);
            // 设置机构信息
            CmClubRemarks clubRemarksInfo = userMapper.getClubRemarksInfo(clubId);
            if (null != clubRemarksInfo) {
                clubStatistics.setCustomerSource(clubRemarksInfo.getCustomerSource() != null ? clubRemarksInfo.getCustomerSource() : 4);
                clubStatistics.setGroupAddition(clubRemarksInfo.getGroupAddition() != null ? clubRemarksInfo.getGroupAddition() : 1);
                clubStatistics.setMobileReach(1);
                if (null != clubRemarksInfo.getCommunicationMethods() && clubRemarksInfo.getCommunicationMethods().contains("1")) {
                    clubStatistics.setMobileReach(0);
                }
                clubStatistics.setWxReach(1);
                if (null != clubRemarksInfo.getCommunicationMethods() && clubRemarksInfo.getCommunicationMethods().contains("2")) {
                    clubStatistics.setWxReach(0);
                }
                clubStatistics.setSex(clubRemarksInfo.getCustomerGender() != null ? clubRemarksInfo.getGroupAddition() : null);
                clubStatistics.setAge(StringUtils.isNotBlank(clubRemarksInfo.getCustomerAge()) ? clubRemarksInfo.getCustomerAge() : null);
                clubStatistics.setFillTime(dateFormat.format(clubRemarksInfo.getAddTime()));
                try {
                    clubStatistics.setFirstTime(StringUtils.isNotBlank(clubRemarksInfo.getFirstTime()) ? dateFormat.format(dateFormat.parse(clubRemarksInfo.getFirstTime())) : null);
                    clubStatistics.setRecentlyTime(StringUtils.isNotBlank(clubRemarksInfo.getRecentlyTime()) ? dateFormat.format(dateFormat.parse(clubRemarksInfo.getRecentlyTime())) : null);
                } catch (ParseException e) {
                    e.printStackTrace();
                }

            }
        }
        return ResponseJson.success(clubStatistics);
    }

    @Override
    public ResponseJson<Map<String, Object>> selPortraitData(CmPortrait cmPortrait) {
        List<String> labelList = new ArrayList<>();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        String endTime = "";
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        Date time = calendar.getTime();
        endTime = dateFormat.format(time);
        String time1 = "";
        String time2 = "";
        boolean expty = ObjectUtils.isEmpty(cmPortrait.getStartTime()) && ObjectUtils.isEmpty(cmPortrait.getEndTime());
        if (expty) {
            Date start = null;
            String startTime = "";
            cmPortrait.setEndTime(endTime);
            // 日期参数为    日
            if (1 == cmPortrait.getDateType() || 1 == cmPortrait.getDateType()) {
                cmPortrait.setStartTime(endTime);
            }
            if (2 == cmPortrait.getDateType()) {
                calendar.setTime(time);
                calendar.add(Calendar.MONTH, -1);
                start = calendar.getTime();
                startTime = dateFormat.format(start);
                cmPortrait.setStartTime(startTime);
            }
            if (3 == cmPortrait.getDateType()) {
                calendar.setTime(time);
                calendar.add(Calendar.MONTH, -6);
                start = calendar.getTime();
                startTime = dateFormat.format(start);
                cmPortrait.setStartTime(startTime);
            }
            if (4 == cmPortrait.getDateType()) {
                calendar.setTime(time);
                calendar.add(Calendar.YEAR, -1);
                start = calendar.getTime();
                startTime = dateFormat.format(start);
                cmPortrait.setStartTime(startTime);
            }
        } else {
            time1 = cmPortrait.getStartTime();
            time2 = cmPortrait.getEndTime();
        }
        // 机构订单
        if (0 == cmPortrait.getType() || 4 == cmPortrait.getType()) {
            if ((0 == cmPortrait.getDateType() || 1 == cmPortrait.getDateType()) && expty) {
                cmPortrait.setEndTime(dateFormat.format(date));
            }
            if (cmPortrait.getEndTime().equals(endTime)) {
                cmPortrait.setEndTime(dateFormat.format(date));
            }
            int ordinary = 0, secondHand = 0, rebate = 0, partialRefund = 0, total = 0; //, fullRefund = 0, closed = 0
            double orderScopeTotalFee = 0.0;
            ArrayList<OrderPortrait> orderPortraits = new ArrayList<>();
            OrderPortrait orderPortrait = new OrderPortrait();
            List<CmShopOrder> orderList = userMapper.selOrderList(cmPortrait);
            int length = orderList.size();
            NumberFormat instance = NumberFormat.getInstance();
            instance.setMaximumFractionDigits(2);
            if (length != 0) {
                for (CmShopOrder order : orderList) {
                    // 普通订单
                    if (("0".equals(order.getSecondHandOrderFlag()) || StringUtils.isEmpty(order.getSecondHandOrderFlag()))
                            && (!"1".equals(order.getRebateFlag()) || "".equals(order.getRebateFlag()) || StringUtils.isEmpty(order.getRebateFlag()))
                            && (!"2".equals(order.getRefundType()) || "".equals(order.getRefundType()) || StringUtils.isEmpty(order.getRefundType()))
                            && !"4".equals(order.getStatus())) {
                        ordinary++;
                    }
                    // 二手订单
                    if ("1".equals(order.getSecondHandOrderFlag()) && !"1".equals(order.getRebateFlag())
                            && (!"2".equals(order.getRefundType())) && !"4".equals(order.getStatus())) {
                        secondHand++;
                    }
                    orderScopeTotalFee += order.getPayTotalFee();
                }
                cmPortrait.setOrderTotal(length);
                // 当前单位订单总额
                cmPortrait.setOrderScopeTotalFee(Double.valueOf(String.format("%.2f", orderScopeTotalFee)));
                orderPortrait.setValue(ordinary);
                if (ordinary != 0) {
                    orderPortrait.setName("普通订单 【" + instance.format((float) ordinary / (float) length * 100) + "%】 【" + ordinary + "个】");
                } else {
                    orderPortrait.setName("普通订单 【 0 %】 【" + ordinary + "个】");
                }
                orderPortraits.add(orderPortrait);
                orderPortrait = new OrderPortrait();
                orderPortrait.setValue(secondHand);
                if (secondHand != 0) {
                    orderPortrait.setName("二手订单 【" + instance.format((float) secondHand / (float) length * 100) + "%】 【" + secondHand + "个】");
                } else {
                    orderPortrait.setName("二手订单 【 0 %】 【" + secondHand + "个】");
                }
                orderPortraits.add(orderPortrait);
                cmPortrait.setOrderPortrait(orderPortraits);
            }
            // 总量
            total = ordinary + secondHand + rebate + partialRefund; //  + fullRefund + closed
            cmPortrait.setOrderScopeNum(total);
            calendar.setTime(date);
            calendar.add(Calendar.DAY_OF_MONTH, -1);
            time = calendar.getTime();
            endTime = dateFormat.format(time);
            cmPortrait.setEndTime(endTime);
        }
        // 搜索关键词
        if (1 == cmPortrait.getType() || 4 == cmPortrait.getType()) {
            List<CmBehaviorRecord> behaviorRecordList = userMapper.selBehaviorList(cmPortrait);
            int keywordsTotal = 0;
            ArrayList<Integer> integers = new ArrayList<>();
            ArrayList<String> strings = new ArrayList<>();
            if (behaviorRecordList.size() != 0) {
                for (CmBehaviorRecord record : behaviorRecordList) {
                    keywordsTotal += record.getNumber();
                    integers.add(record.getNumber());
                    if (StringUtils.isNotBlank(record.getPageLabel())) {
                        labelList = new ArrayList<>();
                        String label = record.getPageLabel();
                        if (label.endsWith(",")) {
                            label = record.getPageLabel().substring(0, record.getPageLabel().lastIndexOf(","));
                        }
                        if (label.startsWith(",")) {
                            label = record.getPageLabel().substring(record.getPageLabel().indexOf(","));
                        }
                        strings.add(label);
                    } else {
                        strings.add("未知");
                    }
                    cmPortrait.setTotalkeywords(behaviorRecordList.get(0).getNumbers());
                    cmPortrait.setBehaviorValue(integers);
                    cmPortrait.setBehaviorType(strings);

                }
            }
            cmPortrait.setKeyWordsScopeNum(keywordsTotal);
        }
        // 咨询记录
        if (2 == cmPortrait.getType() || 4 == cmPortrait.getType()) {
            if ((0 == cmPortrait.getDateType() || 1 == cmPortrait.getDateType()) && expty) {
                cmPortrait.setEndTime(dateFormat.format(date));
            }
            if (cmPortrait.getEndTime().equals(endTime)) {
                cmPortrait.setEndTime(dateFormat.format(date));
            }
            List<CmClubRemarks> remarksList = userMapper.selRemarksList(cmPortrait);
            ArrayList<Integer> integers = new ArrayList<>();
            ArrayList<String> strings = new ArrayList<>();
            ArrayList<String> strRemark = new ArrayList<>();
            ArrayList<String> remarkAll = new ArrayList<>();
            int number = 0;
            if (remarksList.size() != 0) {
                for (CmClubRemarks remarks : remarksList) {
                    String[] split = remarks.getRemarks().split(",");
                    for (String str : split) {
                        if (!strRemark.contains(str)) {
                            strRemark.add(str);
                        }
                        remarkAll.add(str);
                    }
                }
                for (String str : strRemark) {
                    for (int i = 0; i < remarkAll.size(); i++) {
                        if (str.equals(remarkAll.get(i))) {
                            number++;
                        }
                    }
                    integers.add(number);
                    if (!ObjectUtils.isEmpty(str)) {
                        strings.add(str);
                    } else {
                        strings.add("未知");
                    }
                    number = 0;
                }
                // 数组根据出现次数排序
                int temp = 0;
                String strTemp = "";
                for (int i = 0; i < integers.size() - 1; i++) {
                    for (int k = 0; k < integers.size() - 1 - i; k++) {
                        if (integers.get(k) > integers.get(k + 1)) {
                            temp = integers.get(k);
                            integers.set(k, integers.get(k + 1));
                            integers.set(k + 1, temp);

                            strTemp = strings.get(k);
                            strings.set(k, strings.get(k + 1));
                            strings.set(k + 1, strTemp);
                        }
                    }
                }
                cmPortrait.setRemarksValue(integers);
                cmPortrait.setRemarksType(strings);
            }
            cmPortrait.setRemarksScopeNum(remarksList.size());
            calendar.setTime(date);
            calendar.add(Calendar.DAY_OF_MONTH, -1);
            time = calendar.getTime();
            endTime = dateFormat.format(time);
            cmPortrait.setEndTime(endTime);
        }
        // 访问记录
        if (3 == cmPortrait.getType() || 4 == cmPortrait.getType()) {
            List<CmBehaviorRecord> pageTypeList = userMapper.selBehaviorPageTypeList(cmPortrait);
            ArrayList<Integer> integers = new ArrayList<>();
            ArrayList<String> strings = new ArrayList<>();
            Integer number = 0;
            if (pageTypeList.size() != 0) {
                for (CmBehaviorRecord record : pageTypeList) {
                    number += record.getNumbers();
                    if (StringUtils.isNotBlank(record.getPageLabel())) {
                        labelList = new ArrayList<>();
                        String label = record.getPageLabel();
                        integers.add(record.getNumbers());
                        if (label.endsWith(",")) {
                            label = record.getPageLabel().substring(0, record.getPageLabel().lastIndexOf(","));
                        }
                        if (label.startsWith(",")) {
                            label = record.getPageLabel().substring(record.getPageLabel().indexOf(","));
                        }
                        strings.add(label);
                    }
                }
                cmPortrait.setPageTypeValue(integers);
                cmPortrait.setPageType(strings);
                cmPortrait.setVisitNum(number);
            }
        }
        if (!expty) {
            cmPortrait.setStartTime(time1);
            if ((0 == cmPortrait.getDateType() || 1 == cmPortrait.getDateType()) && ObjectUtils.isEmpty(cmPortrait.getStartTime())) {
                cmPortrait.setStartTime(time2);
            }
            cmPortrait.setEndTime(time2);
        }
        Map<String, Object> map = Maps.newLinkedHashMap();
        ArrDate order = new ArrDate();
        ArrDate behavior = new ArrDate();
        ArrDate remarks = new ArrDate();
        ArrDate pageType = new ArrDate();

        order.setOrderList(cmPortrait.getOrderPortrait());
        if (cmPortrait.getOrderTotal() == null) {
            cmPortrait.setOrderTotal(0);
        }
        order.setOrderTotalNum(cmPortrait.getOrderTotal());
        order.setStartTime(cmPortrait.getStartTime());
        order.setEndTime(cmPortrait.getEndTime());
        if (order.getOrderTotalNum() == 0) {
            map.put("order", null);
        } else {
            map.put("order", order);
        }
        behavior.setName(cmPortrait.getBehaviorType());
        behavior.setValue(cmPortrait.getBehaviorValue());
        behavior.setStartTime(cmPortrait.getStartTime());
        behavior.setEndTime(cmPortrait.getEndTime());
        if (ObjectUtils.isEmpty(behavior.getValue())) {
            map.put("behavior", null);
        } else {
            map.put("behavior", behavior);
        }
        remarks.setName(cmPortrait.getRemarksType());
        remarks.setValue(cmPortrait.getRemarksValue());
        remarks.setStartTime(cmPortrait.getStartTime());
        remarks.setEndTime(cmPortrait.getEndTime());
        if (ObjectUtils.isEmpty(remarks.getValue())) {
            map.put("remarks", null);
        } else {
            map.put("remarks", remarks);
        }
        pageType.setName(cmPortrait.getPageType());
        pageType.setValue(cmPortrait.getPageTypeValue());
        pageType.setStartTime(cmPortrait.getStartTime());
        pageType.setEndTime(cmPortrait.getEndTime());
        if (ObjectUtils.isEmpty(pageType.getValue())) {
            map.put("pageType", null);
        } else {
            map.put("pageType", pageType);
        }
        map.put("portrait", cmPortrait);
        return ResponseJson.success(map);
    }

    public void setClubNum(CmPortrait clubStatistics) {
        Integer number = 0;
        if (null != clubStatistics) {
            // 计算资料完整度
            if (StringUtils.isNotBlank(clubStatistics.getName())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getLinkMan())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getContractMobile())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getLinkManIdentity())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getAddress())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getBusinessLicenseImage())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getUserIdentity())) {
                number += 10;
            }
            if (StringUtils.isNotBlank(clubStatistics.getSname())) {
                number += 3;
            }
            if (StringUtils.isNotBlank(clubStatistics.getContractEmail())) {
                number += 5;
            }
            if (StringUtils.isNotBlank(clubStatistics.getContractPhone())) {
                number += 4;
            }
            if (StringUtils.isNotBlank(clubStatistics.getFax())) {
                number += 3;
            }
            if (StringUtils.isNotBlank(clubStatistics.getHeadpic())) {
                number += 5;
            }
            if (StringUtils.isNotBlank(clubStatistics.getSocialCreditCode())) {
                number += 5;
            }
            if (StringUtils.isNotBlank(clubStatistics.getInfo())) {
                number += 5;
            }
        }
        clubStatistics.setNumber(number);
        clubStatistics.setClubDataIntegrity(number + "%");
    }
}
